@@ -22,13 +22,12 @@
 
 #include <config.h>
 #include <string.h>
-#include <gnome.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
+#include <glib/gi18n.h>
 #include "goo-player-info.h"
 #include "goo-marshal.h"
-#include "glib-utils.h"
 #include "goo-stock.h"
-#include "file-utils.h"
+#include "goo-window.h"
+#include "glib-utils.h"
 
 #define SPACING 0
 #define TITLE1_FORMAT "<span size='large' weight='bold'>%s</span>"
@@ -59,9 +58,8 @@ struct _GooPlayerInfoPrivateData {
 	GtkWidget   *cover_button;
 	GtkWidget   *status_image;
 	GtkWidget   *notebook;
-	GtkTooltips *tips;
-	char         current_time[64];
-	char         total_time[64];
+	char        *current_time;
+	char        *total_time;
 	char         time[64];
 	gint64       track_length;
 	gboolean     dragging;
@@ -77,7 +75,6 @@ enum {
         LAST_SIGNAL
 };
 
-static GtkHBoxClass *parent_class = NULL;
 static guint goo_player_info_signals[LAST_SIGNAL] = { 0 };
 
 enum { TARGET_URL };
@@ -86,38 +83,12 @@ static GtkTargetEntry target_table[] = {
 };
 static guint n_targets = sizeof (target_table) / sizeof (target_table[0]);
 
-static void goo_player_info_class_init  (GooPlayerInfoClass *class);
-static void goo_player_info_init        (GooPlayerInfo *player);
 static void goo_player_info_finalize    (GObject *object);
 
+#define GOO_PLAYER_INFO_GET_PRIVATE_DATA(object) \
+	(G_TYPE_INSTANCE_GET_PRIVATE ((object), GOO_TYPE_PLAYER_INFO, GooPlayerInfoPrivateData))
 
-GType
-goo_player_info_get_type ()
-{
-        static GType type = 0;
-
-        if (! type) {
-                GTypeInfo type_info = {
-			sizeof (GooPlayerInfoClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) goo_player_info_class_init,
-			NULL,
-			NULL,
-			sizeof (GooPlayerInfo),
-			0,
-			(GInstanceInitFunc) goo_player_info_init
-		};
-
-		type = g_type_register_static (GTK_TYPE_HBOX,
-					       "GooPlayerInfo",
-					       &type_info,
-					       0);
-	}
-
-        return type;
-}
-
+G_DEFINE_TYPE (GooPlayerInfo, goo_player_info, GTK_TYPE_HBOX)
 
 static void
 goo_player_info_size_request (GtkWidget      *widget,
@@ -125,8 +96,8 @@ goo_player_info_size_request (GtkWidget      *widget,
 {	
 	GooPlayerInfo *info;
 	
-	if (GTK_WIDGET_CLASS (parent_class)->size_request)
-		(* GTK_WIDGET_CLASS (parent_class)->size_request) (widget, requisition);
+	if (GTK_WIDGET_CLASS (goo_player_info_parent_class)->size_request)
+		(* GTK_WIDGET_CLASS (goo_player_info_parent_class)->size_request) (widget, requisition);
 	
 	info = (GooPlayerInfo *) widget;
 	if (info->priv->interactive)
@@ -142,8 +113,6 @@ goo_player_info_class_init (GooPlayerInfoClass *class)
         GObjectClass   *gobject_class;
 	GtkWidgetClass *widget_class;
 
-        parent_class = g_type_class_peek_parent (class);
-
 	goo_player_info_signals[COVER_CLICKED] =
                 g_signal_new ("cover_clicked",
 			      G_TYPE_FROM_CLASS (class),
@@ -169,6 +138,8 @@ goo_player_info_class_init (GooPlayerInfoClass *class)
 
 	widget_class = GTK_WIDGET_CLASS (class);
 	widget_class->size_request = goo_player_info_size_request;
+
+	g_type_class_add_private (class, sizeof (GooPlayerInfoPrivateData));
 }
 
 
@@ -243,7 +214,9 @@ time_scale_value_changed_cb (GtkRange      *range,
 	
 	new_value = gtk_range_get_value (range);
 	current_time = info->priv->track_length * (new_value / 100.0);
-	set_time_string (info->priv->current_time, current_time);
+	g_free (info->priv->current_time);
+	info->priv->current_time = _g_format_duration_for_display (current_time * 1000);
+	/* translators: this is the current_time / total_time label */
 	sprintf (info->priv->time, _("%s / %s"), info->priv->total_time, info->priv->current_time);
 	set_time (info, info->priv->time);
 
@@ -270,7 +243,8 @@ update_time_label_cb (gpointer data)
 	}
 	
 	current_time = priv->track_length * new_value;
-	set_time_string (priv->current_time, current_time);
+	g_free (info->priv->current_time);
+	info->priv->current_time = _g_format_duration_for_display (current_time * 1000);
 	sprintf (priv->time, _("%s / %s"), priv->current_time, priv->total_time);
 	set_time (info, priv->time);
 
@@ -321,82 +295,6 @@ cover_button_clicked_cb (GtkWidget     *button,
 /* -- drag and drop -- */
 
 
-static char *
-get_path_from_url (char *url)
-{
-	GnomeVFSURI *uri;
-	char        *escaped;
-	char        *path;
-
-	if (url == NULL)
-		return NULL;
-
-	uri = gnome_vfs_uri_new (url);
-	escaped = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
-	path = gnome_vfs_unescape_string (escaped, NULL);
-
-	gnome_vfs_uri_unref (uri);
-	g_free (escaped);
-
-	return path;
-}
-
-
-static GList *
-get_file_list_from_dnd_data (gchar *url_list)
-{
-	GList    *list = NULL;
-	int       i;
-	char     *url_start, *url_end;
-	gboolean  dnd_valid = FALSE;
-
-	i = 0;
-	url_start = url_list;
-	while (url_list[i] != '\0')	{
-		char *url;
-
-		while ((url_list[i] != '\0')
-		       && (url_list[i] != '\r')
-		       && (url_list[i] != '\n')) i++;
-
-		url_end = url_list + i;
-		if (strncmp (url_start, "file:", 5) == 0) {
-			url_start += 5;
-			if ((url_start[0] == '/') 
-			    && (url_start[1] == '/')) url_start += 2;
-
-			dnd_valid = TRUE;
-
-			url = g_strndup (url_start, url_end - url_start);
-			list = g_list_prepend (list, get_path_from_url (url));
-			g_free (url);
-
-			while ((url_list[i] != '\0')
-			       && ((url_list[i] == '\r') 
-				   || (url_list[i] == '\n'))) i++;
-			url_start = url_list + i;
-
-		} else {
-			while ((url_list[i] != '\0')
-			       && ((url_list[i] == '\r') 
-				   || (url_list[i] == '\n'))) i++;
-			url_start = url_list + i;
-			if (url_list[i] == '\0' && !dnd_valid) {
-				path_list_free (list);
-				return NULL;
-			}
-		}
-	}
-	
-	if (!dnd_valid) {
-		path_list_free (list);
-		return NULL;
-	}
-
-	return g_list_reverse (list);
-}
-
-
 void  
 cover_button_drag_data_received  (GtkWidget          *widget,
 				  GdkDragContext     *context,
@@ -407,9 +305,8 @@ cover_button_drag_data_received  (GtkWidget          *widget,
 				  guint               time,
 				  gpointer            extra_data)
 {
-	GooPlayerInfo *info = extra_data;
-	GList         *list;
-	char          *cover_filename;
+	GooPlayerInfo  *info = extra_data;
+	char          **uris;
 
 	if (! ((data->length >= 0) && (data->format == 8))) {
 		gtk_drag_finish (context, FALSE, FALSE, time);
@@ -418,28 +315,33 @@ cover_button_drag_data_received  (GtkWidget          *widget,
 
 	gtk_drag_finish (context, TRUE, FALSE, time);
 
-	list = get_file_list_from_dnd_data ((char *)data->data);
-	if (list == NULL)
-		return;
+	uris = gtk_selection_data_get_uris (data);
+	if (uris[0] != NULL) {
+		GFile *file;
+		char  *cover_filename;
 
-	cover_filename = list->data;
-	if (cover_filename == NULL)
-		return;
+		file = g_file_new_for_uri (uris[0]);
+		cover_filename = g_file_get_path (file);
+		goo_window_set_cover_image (info->priv->window, cover_filename);
 
-	goo_window_set_cover_image (info->priv->window, cover_filename);
+		g_free (cover_filename);
+		g_object_unref (file);
+	}
 
-	if (list != NULL) 
-		path_list_free (list);
+	g_strfreev (uris);
 }
 
 
 static void 
 goo_player_info_init (GooPlayerInfo *info)
 {
-	info->priv = g_new0 (GooPlayerInfoPrivateData, 1);
+	info->priv = GOO_PLAYER_INFO_GET_PRIVATE_DATA (info);
 }
 
 
+static void goo_player_info_update_state (GooPlayerInfo *info);
+
+
 static void 
 goo_player_info_construct (GooPlayerInfo *info)
 {
@@ -449,18 +351,14 @@ goo_player_info_construct (GooPlayerInfo *info)
 	priv = info->priv;
 
 	priv->dragging = FALSE;
-	priv->current_time[0] = '\0';
-	priv->total_time[0] = '\0';
+	priv->current_time = NULL;
+	priv->total_time = NULL;
 	priv->update_id = 0;
 
 	GTK_WIDGET_UNSET_FLAGS (info, GTK_CAN_FOCUS);
 	GTK_BOX (info)->spacing = SPACING;
 	GTK_BOX (info)->homogeneous = FALSE;
 
-	priv->tips = gtk_tooltips_new ();
-	gtk_object_ref (GTK_OBJECT (priv->tips));
-	gtk_object_sink (GTK_OBJECT (priv->tips));
-
 	/* Title and Artist */
 
 	priv->title1_label = gtk_label_new (NULL);
@@ -515,10 +413,9 @@ goo_player_info_construct (GooPlayerInfo *info)
 		gtk_button_set_relief (GTK_BUTTON (priv->cover_button),
 				       GTK_RELIEF_NONE);
 
-		gtk_tooltips_set_tip (priv->tips,
-				      GTK_WIDGET (priv->cover_button),
-				      _("Click here to choose a cover for this CD"),
-				      NULL);
+		gtk_widget_set_tooltip_text (GTK_WIDGET (priv->cover_button),
+					     _("Click here to choose a cover for this CD"));
+
 		g_signal_connect (G_OBJECT (priv->cover_button),
 				  "clicked",
 				  G_CALLBACK (cover_button_clicked_cb),
@@ -605,6 +502,8 @@ goo_player_info_construct (GooPlayerInfo *info)
 			  "button_release_event",
 			  G_CALLBACK (time_scale_button_release_cb), 
 			  info);
+
+	goo_player_info_update_state (info);
 }
 
 
@@ -618,16 +517,16 @@ goo_player_info_finalize (GObject *object)
   
 	info = GOO_PLAYER_INFO (object);
 	if (info->priv != NULL) {
+		g_free (info->priv->current_time);
+		g_free (info->priv->total_time);
 		if (info->priv->update_id != 0) {
 			g_source_remove (info->priv->update_id);
 			info->priv->update_id = 0;
 		}
-		gtk_object_unref (GTK_OBJECT (info->priv->tips));
-		g_free (info->priv);
 		info->priv = NULL;
 	}
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (goo_player_info_parent_class)->finalize (object);
 }
 
 
@@ -635,18 +534,21 @@ static void
 goo_player_info_set_time (GooPlayerInfo  *info,
 			  gint64          current_time)
 {
-	GooPlayerInfoPrivateData *priv = info->priv;
-
-	if (priv->dragging) 
+	if (info->priv->dragging)
 		return;
 
-	set_time_string (priv->current_time, current_time);
-	sprintf (priv->time, _("%s / %s"), priv->current_time, priv->total_time);
-	set_time (info, priv->time);
+	g_free (info->priv->current_time);
+	info->priv->current_time = _g_format_duration_for_display (current_time * 1000);
+	sprintf (info->priv->time, _("%s / %s"), info->priv->current_time, info->priv->total_time);
+	set_time (info, info->priv->time);
 
-	g_signal_handlers_block_by_data (priv->time_scale, info);
-	gtk_range_set_value (GTK_RANGE (priv->time_scale), (double) current_time / priv->track_length);
-	g_signal_handlers_unblock_by_data (priv->time_scale, info);
+	g_signal_handlers_block_by_data (info->priv->time_scale, info);
+	gtk_range_set_value (GTK_RANGE (info->priv->time_scale), (double) current_time / info->priv->track_length);
+	g_signal_handlers_unblock_by_data (info->priv->time_scale, info);
+
+	/* FIXME: this doesn't update the tooltip
+	gtk_tooltip_trigger_tooltip_query (gdk_screen_get_display (gtk_status_icon_get_screen (goo_window_get_status_icon (info->priv->window))));
+	*/
 }
 
 
@@ -690,7 +592,8 @@ update_subtitle (GooPlayerInfo *info,
 	album = goo_window_get_album (info->priv->window);
 	
 	if ((album->title == NULL) || (album->artist == NULL)) {
-		set_time_string (info->priv->total_time, track->length);
+		g_free (info->priv->total_time);
+		info->priv->total_time = _g_format_duration_for_display (track->length * 1000);
 		set_title2 (info, info->priv->total_time);
 	} 
 	else {
@@ -703,7 +606,7 @@ update_subtitle (GooPlayerInfo *info,
 
 
 static void
-goo_player_info_update_state (GooPlayerInfo  *info)
+goo_player_info_update_state (GooPlayerInfo *info)
 {
 	GooPlayerInfoPrivateData *priv = info->priv;
 	GooPlayerState  state;
@@ -736,16 +639,17 @@ goo_player_info_update_state (GooPlayerInfo  *info)
 	gtk_label_set_selectable (GTK_LABEL (priv->title2_label), FALSE);
 	gtk_label_set_selectable (GTK_LABEL (priv->title3_label), FALSE);
 
-	if ((state == GOO_PLAYER_STATE_ERROR)
-	    || (state == GOO_PLAYER_STATE_NO_DISC)
-	    || (state == GOO_PLAYER_STATE_DATA_DISC)) {
-		GError *error = goo_player_get_error (player);
-		
-		set_title1 (info, error->message);
+	if ((state == GOO_PLAYER_STATE_ERROR) || (state == GOO_PLAYER_STATE_NO_DISC))
+	{
+		set_title1 (info, _("No disc"));
 		set_title2 (info, "");
 		set_title3 (info, "");
-		g_error_free (error);
 	} 
+	else if (state == GOO_PLAYER_STATE_DATA_DISC) {
+		set_title1 (info, _("Data disc"));
+		set_title2 (info, "");
+		set_title3 (info, "");
+	}
 	else {
 		TrackInfo *track;
 		
@@ -844,7 +748,8 @@ goo_player_info_set_total_time (GooPlayerInfo  *info,
 {
 	GooPlayerInfoPrivateData *priv = info->priv;
 
-	set_time_string (priv->total_time, total_time);
+	g_free (priv->total_time);
+	priv->total_time = _g_format_duration_for_display (total_time * 1000);
 	goo_player_info_update_state (info);
 }
 
@@ -873,7 +778,7 @@ player_done_cb (GooPlayer       *player,
 		break;
 	case GOO_PLAYER_ACTION_PLAY:
 	case GOO_PLAYER_ACTION_STOP:
-	case GOO_PLAYER_ACTION_EJECT:
+	case GOO_PLAYER_ACTION_MEDIUM_REMOVED:
 		goo_player_info_set_time (info, 0);
 		break;
 	default:
@@ -1000,3 +905,13 @@ goo_player_info_new (GooWindow *window,
 
 	return GTK_WIDGET (info);
 }
+
+
+GdkPixbuf *
+goo_player_info_get_cover (GooPlayerInfo *info)
+{
+	if (gtk_image_get_storage_type (GTK_IMAGE (info->priv->cover_image)) == GTK_IMAGE_PIXBUF)
+		return gtk_image_get_pixbuf (GTK_IMAGE (info->priv->cover_image));
+	else
+		return NULL;
+}
