@@ -39,16 +39,14 @@
 
 #include <config.h>
 #include <math.h>
-#include <gnome.h>
 #include <gdk/gdkkeysyms.h>
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 #include "glib-utils.h"
 #include "goo-volume-tool-button.h"
 #include "goo-marshal.h"
 #include "goo-stock.h"
 
-#define GOO_VOLUME_TOOL_BUTTON_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), GOO_TYPE_VOLUME_TOOL_BUTTON, GooVolumeToolButtonPrivate))
-#define FLOAT_EQUAL(a,b) (fabs ((a) - (b)) < 1e-6)
-
 #define SCALE_WIDTH 25
 #define SCALE_HEIGHT 100
 #define DEFAULT_VOLUME 1.0
@@ -64,7 +62,6 @@ struct _GooVolumeToolButtonPrivate {
 	GtkWidget   *popup_win;
 	GtkWidget   *volume_scale;
 	GtkWidget   *volume_label;
-	GtkTooltips *tips;
 	double       value;
 	double       from_value;
 	double       to_value;
@@ -86,75 +83,57 @@ enum {
         LAST_SIGNAL
 };
 
-static GObjectClass *parent_class = NULL;
 static guint goo_volume_tool_button_signals[LAST_SIGNAL] = { 0 };
 
 
-static gboolean
-goo_volume_tool_button_set_tooltip (GtkToolItem *tool_item,
-				    GtkTooltips *tooltips,
-				    const char  *tip_text,
-				    const char  *tip_private)
-{
-	GooVolumeToolButton *button;
-
-	g_return_val_if_fail (GOO_IS_VOLUME_TOOL_BUTTON (tool_item), FALSE);
-
-	button = GOO_VOLUME_TOOL_BUTTON (tool_item);
-	/* gtk_tooltips_set_tip (tooltips, button->priv->button, tip_text, tip_private);*/
-	
-	return TRUE;
-}
+G_DEFINE_TYPE (GooVolumeToolButton, goo_volume_tool_button, GTK_TYPE_TOOL_BUTTON)
 
 
 static void
 goo_volume_tool_button_construct_contents (GooVolumeToolButton *button)
 {
-	GooVolumeToolButtonPrivate *priv;
-	GtkWidget                  *box;
-	GtkOrientation              orientation;
-
-	priv = GOO_VOLUME_TOOL_BUTTON_GET_PRIVATE (button);
+	GtkWidget      *box;
+	GtkOrientation  orientation;
 
 	orientation = gtk_tool_item_get_orientation (GTK_TOOL_ITEM (button));
 
 	if (orientation == GTK_ORIENTATION_HORIZONTAL) {
 		box = gtk_hbox_new (FALSE, 0);
-		gtk_arrow_set (GTK_ARROW (priv->arrow), GTK_ARROW_DOWN, GTK_SHADOW_NONE);
+		gtk_arrow_set (GTK_ARROW (button->priv->arrow), GTK_ARROW_DOWN, GTK_SHADOW_NONE);
 	} 
 	else {
 		box = gtk_vbox_new (FALSE, 0);
-		gtk_arrow_set (GTK_ARROW (priv->arrow), GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
+		gtk_arrow_set (GTK_ARROW (button->priv->arrow), GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
 	}
 
-	if (priv->button && priv->button->parent) {
-		g_object_ref (priv->button);
-		gtk_container_remove (GTK_CONTAINER (priv->button->parent),
-				      priv->button);
-		gtk_container_add (GTK_CONTAINER (box), priv->button);
-		g_object_unref (priv->button);
+	if (button->priv->button && button->priv->button->parent) {
+		g_object_ref (button->priv->button);
+		gtk_container_remove (GTK_CONTAINER (button->priv->button->parent),
+				      button->priv->button);
+		gtk_container_add (GTK_CONTAINER (box), button->priv->button);
+		g_object_unref (button->priv->button);
 	}
 
-	if (priv->arrow_button && priv->arrow_button->parent) {
-		g_object_ref (priv->arrow_button);
-		gtk_container_remove (GTK_CONTAINER (priv->arrow_button->parent),
-				      priv->arrow_button);
-		gtk_box_pack_end (GTK_BOX (box), priv->arrow_button,
+	if (button->priv->arrow_button && button->priv->arrow_button->parent) {
+		g_object_ref (button->priv->arrow_button);
+		gtk_container_remove (GTK_CONTAINER (button->priv->arrow_button->parent),
+				      button->priv->arrow_button);
+		gtk_box_pack_end (GTK_BOX (box), button->priv->arrow_button,
 				  FALSE, FALSE, 0);
-		g_object_unref (priv->arrow_button);
+		g_object_unref (button->priv->arrow_button);
 	}
 
-	if (priv->box) {
+	if (button->priv->box) {
 		/* Note: we are not destroying the button and the arrow_button
 		 * here because they were removed from their container above
 		 */
-		gtk_widget_destroy (priv->box);
+		gtk_widget_destroy (button->priv->box);
 	}
 
-	priv->box = box;
+	button->priv->box = box;
 
-	gtk_container_add (GTK_CONTAINER (button), priv->box);
-	gtk_widget_show_all (priv->box);
+	gtk_container_add (GTK_CONTAINER (button), button->priv->box);
+	gtk_widget_show_all (button->priv->box);
 
 	gtk_widget_queue_resize (GTK_WIDGET (button));
 }
@@ -183,10 +162,8 @@ update_volume_label (GooVolumeToolButton *button)
 	g_free (text);
 
 	text = g_strdup_printf (_("Volume level: %3.0f%%"), value * 100.0);
-	gtk_tooltips_set_tip (button->priv->tips,
-			      button->priv->button,
-			      text,
-			      NULL);
+	gtk_widget_set_tooltip_text (button->priv->button, text);
+
 	g_free (text);
 }
 
@@ -197,14 +174,14 @@ goo_volume_tool_button_toolbar_reconfigured (GtkToolItem *toolitem)
 	goo_volume_tool_button_construct_contents (GOO_VOLUME_TOOL_BUTTON (toolitem));
 	
 	/* chain up */
-	GTK_TOOL_ITEM_CLASS (parent_class)->toolbar_reconfigured (toolitem);
+	GTK_TOOL_ITEM_CLASS (goo_volume_tool_button_parent_class)->toolbar_reconfigured (toolitem);
 }
 
 
 static void
 goo_volume_tool_button_init (GooVolumeToolButton *button)
 {
-	button->priv = GOO_VOLUME_TOOL_BUTTON_GET_PRIVATE (button);
+	button->priv = G_TYPE_INSTANCE_GET_PRIVATE ((button), GOO_TYPE_VOLUME_TOOL_BUTTON, GooVolumeToolButtonPrivate);
 	button->priv->mute = FALSE;
 	button->priv->mute_value = 0.0;
 	button->priv->value = 0.0;
@@ -218,25 +195,15 @@ goo_volume_tool_button_finalize (GObject *object)
 	GooVolumeToolButton *button;
 
 	button = GOO_VOLUME_TOOL_BUTTON (object);
-	gtk_object_unref (GTK_OBJECT (button->priv->tips));
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (goo_volume_tool_button_parent_class)->finalize (object);
 }
 
 
 static void
 goo_volume_tool_button_class_init (GooVolumeToolButtonClass *klass)
 {
-	GObjectClass       *object_class;
-	GtkToolItemClass   *toolitem_class;
-	GtkToolButtonClass *toolbutton_class;
-	GtkWidgetClass     *gtkwidget_class;
-	
-	parent_class = g_type_class_peek_parent (klass);
-  
-	object_class = (GObjectClass *) klass;
-	toolitem_class = (GtkToolItemClass *) klass;
-	toolbutton_class = (GtkToolButtonClass *) klass;
-	gtkwidget_class = (GtkWidgetClass *) klass;
+	GObjectClass     *object_class;
+	GtkToolItemClass *toolitem_class;
 
 	goo_volume_tool_button_signals[CHANGED] =
                 g_signal_new ("changed",
@@ -248,74 +215,16 @@ goo_volume_tool_button_class_init (GooVolumeToolButtonClass *klass)
 			      G_TYPE_NONE, 
 			      0);
 
+	object_class = (GObjectClass *) klass;
 	object_class->finalize = goo_volume_tool_button_finalize;
-	toolitem_class->set_tooltip = goo_volume_tool_button_set_tooltip;
+
+	toolitem_class = (GtkToolItemClass *) klass;
 	toolitem_class->toolbar_reconfigured = goo_volume_tool_button_toolbar_reconfigured;
 	
 	g_type_class_add_private (object_class, sizeof (GooVolumeToolButtonPrivate));
 }
 
 
-GType
-goo_volume_tool_button_get_type (void)
-{
-	static GType type = 0;
-
-	if (type == 0) {
-		static const GTypeInfo info = {
-			sizeof (GooVolumeToolButtonClass),
-			(GBaseInitFunc) 0,
-			(GBaseFinalizeFunc) 0,
-			(GClassInitFunc) goo_volume_tool_button_class_init,
-			(GClassFinalizeFunc) 0,
-			NULL,
-			sizeof (GooVolumeToolButton),
-			0, /* n_preallocs */
-			(GInstanceInitFunc) goo_volume_tool_button_init
-		};
-		
-		type = g_type_register_static (GTK_TYPE_TOOL_BUTTON,
-					       "GooVolumeToolButton",
-					       &info, 0);
-	}
-	
-	return type;
-}
-
-
-static void
-button_state_changed_cb (GtkWidget           *widget,
-                         GtkStateType         previous_state,
-                         GooVolumeToolButton *button)
-{
-	GooVolumeToolButtonPrivate *priv;
-	GtkWidget *other;
-	GtkStateType state = GTK_WIDGET_STATE (widget);
-	
-	priv = GOO_VOLUME_TOOL_BUTTON_GET_PRIVATE (button);
-
-	other = (widget == priv->arrow_button) ? priv->button : priv->arrow_button;
-
-	g_signal_handlers_block_by_func (other,
-					 G_CALLBACK (button_state_changed_cb),
-					 button);
-	
-	if (state == GTK_STATE_PRELIGHT) {
-		gtk_widget_set_state (other, state);
-	} 
-	else if (state == GTK_STATE_NORMAL) {
-		gtk_widget_set_state (other, state);
-	} 
-	else if (state == GTK_STATE_ACTIVE) {
-		gtk_widget_set_state (other, GTK_STATE_NORMAL);
-	}
-
-	g_signal_handlers_unblock_by_func (other,
-					   G_CALLBACK (button_state_changed_cb),
-					   button);
-}
-
-
 static gboolean
 button_clicked_cb (GtkWidget           *widget,
 		   GooVolumeToolButton *button)
@@ -343,7 +252,6 @@ arrow_button_press_cb (GtkToggleButton     *toggle_button,
 		       GdkEventButton      *event,
 		       GooVolumeToolButton *button)
 {
-	GooVolumeToolButtonPrivate *priv;
 	GtkWidget      *widget;
 	GtkAdjustment  *adj;
   	int             x, y, m, dx, dy, sx, sy, ystartoff, mouse_y;
@@ -351,43 +259,41 @@ arrow_button_press_cb (GtkToggleButton     *toggle_button,
   	GdkEventButton *e;
 
 	gtk_toggle_button_set_active (toggle_button, TRUE);
-  	
-	priv = GOO_VOLUME_TOOL_BUTTON_GET_PRIVATE (button);
 	
 	widget = GTK_WIDGET (toggle_button);
-	adj = gtk_range_get_adjustment (GTK_RANGE (priv->volume_scale));
+	adj = gtk_range_get_adjustment (GTK_RANGE (button->priv->volume_scale));
 	
-	priv->timeout = TRUE;
+	button->priv->timeout = TRUE;
 
-    	gtk_window_set_screen (GTK_WINDOW (priv->popup_win), gtk_widget_get_screen (widget));
+    	gtk_window_set_screen (GTK_WINDOW (button->priv->popup_win), gtk_widget_get_screen (widget));
   
   	gdk_window_get_origin (widget->window, &x, &y);
   	x += widget->allocation.x;
   	y += widget->allocation.y;
   
-	gtk_window_move (GTK_WINDOW (priv->popup_win),
+	gtk_window_move (GTK_WINDOW (button->priv->popup_win),
 			 x, y - (SCALE_HEIGHT / 2));
 	update_volume_label (button);
-	gtk_widget_show_all (priv->popup_win);
+	gtk_widget_show_all (button->priv->popup_win);
 
-	gdk_window_get_origin (priv->popup_win->window, &dx, &dy);
-  	dy += priv->popup_win->allocation.y;
-  	gdk_window_get_origin (priv->volume_scale->window, &sx, &sy);
-  	sy += priv->volume_scale->allocation.y;
+	gdk_window_get_origin (button->priv->popup_win->window, &dx, &dy);
+  	dy += button->priv->popup_win->allocation.y;
+  	gdk_window_get_origin (button->priv->volume_scale->window, &sx, &sy);
+  	sy += button->priv->volume_scale->allocation.y;
   	ystartoff = sy - dy;
   	mouse_y = event->y;
 
-	v = gtk_range_get_value (GTK_RANGE (priv->volume_scale)) / (adj->upper - adj->lower);
-  	x += (widget->allocation.width - priv->popup_win->allocation.width) / 2;
+	v = gtk_range_get_value (GTK_RANGE (button->priv->volume_scale)) / (adj->upper - adj->lower);
+  	x += (widget->allocation.width - button->priv->popup_win->allocation.width) / 2;
   	y -= ystartoff;
-  	y -= GTK_RANGE (priv->volume_scale)->min_slider_size / 2;
-  	m = priv->volume_scale->allocation.height - GTK_RANGE (priv->volume_scale)->min_slider_size;
+  	y -= GTK_RANGE (button->priv->volume_scale)->min_slider_size / 2;
+  	m = button->priv->volume_scale->allocation.height - GTK_RANGE (button->priv->volume_scale)->min_slider_size;
   	y -= m * (1.0 - v);
   	y += mouse_y;
-  	gtk_window_move (GTK_WINDOW (priv->popup_win), x, y);
-  	gdk_window_get_origin (priv->volume_scale->window, &sx, &sy);
+  	gtk_window_move (GTK_WINDOW (button->priv->popup_win), x, y);
+  	gdk_window_get_origin (button->priv->volume_scale->window, &sx, &sy);
 	
-	gdk_pointer_grab (priv->popup_win->window, 
+	gdk_pointer_grab (button->priv->popup_win->window,
 			  TRUE,
 			  (GDK_POINTER_MOTION_MASK 
 			   | GDK_BUTTON_PRESS_MASK 
@@ -395,21 +301,21 @@ arrow_button_press_cb (GtkToggleButton     *toggle_button,
 			  NULL, 
 			  NULL, 
 			  GDK_CURRENT_TIME);
-	gdk_keyboard_grab (priv->popup_win->window, TRUE, GDK_CURRENT_TIME);
-	gtk_widget_grab_focus (priv->volume_scale);
-	gtk_grab_add (priv->popup_win);
+	gdk_keyboard_grab (button->priv->popup_win->window, TRUE, GDK_CURRENT_TIME);
+	gtk_widget_grab_focus (button->priv->volume_scale);
+	gtk_grab_add (button->priv->popup_win);
 
 	/* forward event to the slider */
 	e = (GdkEventButton *) gdk_event_copy ((GdkEvent *) event);
-	e->window = priv->volume_scale->window;
-	e->x = priv->volume_scale->allocation.width / 2;
-	m = priv->volume_scale->allocation.height - GTK_RANGE (priv->volume_scale)->min_slider_size;
-	e->y = ((1.0 - v) * m) + GTK_RANGE (priv->volume_scale)->min_slider_size / 2 + 1;
-	gtk_widget_event (priv->volume_scale, (GdkEvent *) e);
+	e->window = button->priv->volume_scale->window;
+	e->x = button->priv->volume_scale->allocation.width / 2;
+	m = button->priv->volume_scale->allocation.height - GTK_RANGE (button->priv->volume_scale)->min_slider_size;
+	e->y = ((1.0 - v) * m) + GTK_RANGE (button->priv->volume_scale)->min_slider_size / 2 + 1;
+	gtk_widget_event (button->priv->volume_scale, (GdkEvent *) e);
 	e->window = event->window;
 	gdk_event_free ((GdkEvent *) e);		
 
-	priv->pop_time = event->time;
+	button->priv->pop_time = event->time;
 
 	return TRUE;
 }
@@ -447,16 +353,13 @@ scale_button_release_cb (GtkToggleButton     *toggle_button,
 		         GdkEventButton      *event,
 		         GooVolumeToolButton *button)
 {
-	GooVolumeToolButtonPrivate *priv;
-	priv = GOO_VOLUME_TOOL_BUTTON_GET_PRIVATE (button);
-
-	if (priv->timeout) {
+	if (button->priv->timeout) {
 		/* if we did a quick click, leave the window open; else, hide it */
-		if (event->time > priv->pop_time + CLICK_TIMEOUT) {
+		if (event->time > button->priv->pop_time + CLICK_TIMEOUT) {
 			ungrab (button);
 			return FALSE;
 		}
-		priv->timeout = FALSE;
+		button->priv->timeout = FALSE;
 	}
 	
 	return FALSE;
@@ -468,10 +371,7 @@ popup_win_event_cb (GtkWidget           *widget,
 		    GdkEvent            *event, 
 		    GooVolumeToolButton *button)
 {
-	GooVolumeToolButtonPrivate *priv;
-	GtkWidget                  *event_widget;
-
-	priv = GOO_VOLUME_TOOL_BUTTON_GET_PRIVATE (button);
+	GtkWidget *event_widget;
 
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
@@ -485,7 +385,7 @@ popup_win_event_cb (GtkWidget           *widget,
 		else {
 			int x, y, w, h;
 			
-			gdk_window_get_geometry (priv->popup_win->window, 
+			gdk_window_get_geometry (button->priv->popup_win->window,
 						 &x, &y, &w, &h, NULL);
 
 			if ((event->button.x < 0) 
@@ -521,7 +421,6 @@ button_scroll_event_cb (GtkWidget           *widget,
 			GdkEventScroll      *event,
 			GooVolumeToolButton *button)
 {
-	GooVolumeToolButtonPrivate *priv = GOO_VOLUME_TOOL_BUTTON_GET_PRIVATE (button);
 	double direction = 1.0;
 
 	if (event->direction == GDK_SCROLL_UP) 
@@ -532,7 +431,7 @@ button_scroll_event_cb (GtkWidget           *widget,
 		return FALSE;
 
 	goo_volume_tool_button_set_volume (button, 
-					   priv->value + (direction * priv->step),
+					   button->priv->value + (direction * button->priv->step),
 					   TRUE);
 
 	return TRUE;
@@ -710,7 +609,6 @@ up_button_clicked_cb (GtkButton *gtk_button,
 static void 
 goo_volume_button_construct (GooVolumeToolButton *button)
 {
-	struct _GooVolumeToolButtonPrivate *priv = button->priv;
 	GtkWidget *box;
 	GtkWidget *arrow;
 	GtkWidget *arrow_button;
@@ -720,23 +618,19 @@ goo_volume_button_construct (GooVolumeToolButton *button)
 	GtkWidget *up_button;
 	GtkWidget *down_button;
 	
-	priv->tips = gtk_tooltips_new ();
-	gtk_object_ref (GTK_OBJECT (priv->tips));
-	gtk_object_sink (GTK_OBJECT (priv->tips));
-
 	/* Create the popup window. */
 
-	priv->popup_win = gtk_window_new (GTK_WINDOW_POPUP);
-	gtk_window_set_wmclass (GTK_WINDOW (priv->popup_win), "", "goo_volume_button");
+	button->priv->popup_win = gtk_window_new (GTK_WINDOW_POPUP);
+	gtk_window_set_wmclass (GTK_WINDOW (button->priv->popup_win), "", "goo_volume_button");
 	
-	g_signal_connect (G_OBJECT (priv->popup_win),
+	g_signal_connect (G_OBJECT (button->priv->popup_win),
 			  "event",
 			  G_CALLBACK (popup_win_event_cb), 
 			  button);
 
 	out_frame = gtk_frame_new (NULL);
 	gtk_frame_set_shadow_type (GTK_FRAME (out_frame), GTK_SHADOW_OUT);
-	gtk_container_add (GTK_CONTAINER (priv->popup_win), out_frame);
+	gtk_container_add (GTK_CONTAINER (button->priv->popup_win), out_frame);
 
 	/**/
 
@@ -768,20 +662,20 @@ goo_volume_button_construct (GooVolumeToolButton *button)
 			  button);
 	gtk_box_pack_start (GTK_BOX (volume_vbox), up_button, FALSE, FALSE, 0);
 
-	priv->volume_scale = gtk_vscale_new_with_range (0.0, 1.0, 0.1);
-	gtk_range_set_inverted (GTK_RANGE (priv->volume_scale), TRUE);
-	gtk_scale_set_draw_value (GTK_SCALE (priv->volume_scale), FALSE);
-	/*gtk_range_set_update_policy (GTK_RANGE (priv->volume_scale), GTK_UPDATE_DELAYED);*/
-	gtk_range_set_increments (GTK_RANGE (priv->volume_scale), 0.1, 0.1);
-	gtk_widget_set_size_request (priv->volume_scale, -1, SCALE_HEIGHT);
+	button->priv->volume_scale = gtk_vscale_new_with_range (0.0, 1.0, 0.1);
+	gtk_range_set_inverted (GTK_RANGE (button->priv->volume_scale), TRUE);
+	gtk_scale_set_draw_value (GTK_SCALE (button->priv->volume_scale), FALSE);
+	/*gtk_range_set_update_policy (GTK_RANGE (button->priv->volume_scale), GTK_UPDATE_DELAYED);*/
+	gtk_range_set_increments (GTK_RANGE (button->priv->volume_scale), 0.1, 0.1);
+	gtk_widget_set_size_request (button->priv->volume_scale, -1, SCALE_HEIGHT);
 	 
-	gtk_box_pack_start (GTK_BOX (volume_vbox), priv->volume_scale, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (volume_vbox), button->priv->volume_scale, TRUE, TRUE, 0);
 
-	g_signal_connect (priv->volume_scale, 
+	g_signal_connect (button->priv->volume_scale,
 			  "value_changed",
 			  G_CALLBACK (volume_scale_value_changed_cb), 
 			  button);
-	g_signal_connect (priv->volume_scale, 
+	g_signal_connect (button->priv->volume_scale,
 			  "button_release_event",
 			  G_CALLBACK (scale_button_release_cb), 
 			  button);
@@ -813,8 +707,8 @@ goo_volume_button_construct (GooVolumeToolButton *button)
 	/**/
 
 	gtk_box_pack_start (GTK_BOX (volume_vbox), gtk_hseparator_new (), FALSE, FALSE, 0);
-	priv->volume_label = gtk_label_new (NULL);
-	gtk_box_pack_start (GTK_BOX (volume_vbox), priv->volume_label, FALSE, FALSE, 3);
+	button->priv->volume_label = gtk_label_new (NULL);
+	gtk_box_pack_start (GTK_BOX (volume_vbox), button->priv->volume_label, FALSE, FALSE, 3);
 
 	/* Construct the toolbar button.  */
 
@@ -835,7 +729,7 @@ goo_volume_button_construct (GooVolumeToolButton *button)
 	gtk_container_add (GTK_CONTAINER (arrow_button), arrow);
 	gtk_box_pack_end (GTK_BOX (box), arrow_button,
 			  FALSE, FALSE, 0);
-	gtk_tooltips_set_tip (priv->tips, arrow_button, _("Change the volume level"), NULL);
+	gtk_widget_set_tooltip_text (arrow_button, _("Change the volume level"));
 
 	gtk_widget_show_all (box);
 	gtk_widget_hide (button->priv->volume_label);
@@ -847,15 +741,6 @@ goo_volume_button_construct (GooVolumeToolButton *button)
 	button->priv->arrow_button = arrow_button;
 	button->priv->box = box;
 
-	g_signal_connect (real_button, 
-			  "state_changed",
-			  G_CALLBACK (button_state_changed_cb), 
-			  button);
-	g_signal_connect (arrow_button, 
-			  "state_changed",
-			  G_CALLBACK (button_state_changed_cb), 
-			  button);
-
 	g_signal_connect (real_button, 
 			  "clicked",
 			  G_CALLBACK (button_clicked_cb), 
