@@ -25,13 +25,11 @@
 
 #define OGG_ENCODER "vorbisenc"
 #define FLAC_ENCODER "flacenc"
-#define MP3_ENCODER "lame"
 #define WAVE_ENCODER "wavenc"
 
 typedef enum {
 	GOO_FILE_FORMAT_OGG,
 	GOO_FILE_FORMAT_FLAC,
-	GOO_FILE_FORMAT_MP3,
 	GOO_FILE_FORMAT_WAVE
 } GooFileFormat;
 
@@ -43,5 +41,12 @@ typedef enum {
 } WindowSortMethod;
 
 
+typedef void (*DataFunc)         (gpointer    user_data);
+typedef void (*ReadyFunc)        (GError     *error,
+			 	  gpointer    user_data);
+typedef void (*ReadyCallback)    (GObject    *object,
+				  GError     *error,
+			   	  gpointer    user_data);
+
 #endif /* TYPEDEFS_H */
 
