@@ -25,7 +25,6 @@
 
 
 #include <config.h>
-#include <gnome.h>
 #include "actions.h"
 #include "goo-stock.h"
 
@@ -37,7 +36,7 @@ static GtkActionEntry action_entries[] = {
 	{ "HelpMenu", NULL, N_("_Help") },
 	{ "CDCoverMenu", NULL, N_("C_over") },
 
-	{ "About", GNOME_STOCK_ABOUT,
+	{ "About", GTK_STOCK_ABOUT,
 	  NULL, NULL,
 	  N_("Information about the program"),
 	  G_CALLBACK (activate_action_about) },
@@ -110,10 +109,10 @@ static GtkActionEntry action_entries[] = {
 	  N_("Quit the application"),
 	  G_CALLBACK (activate_action_quit) },
 	{ "PickCoverFromDisk", GTK_STOCK_OPEN,
-	  N_("_Choose from Disk"), "",
-	  N_("Choose a disc cover from the local disk"),
+	  N_("_Choose File..."), "",
+	  N_("Choose a file to use as disc cover"),
 	  G_CALLBACK (activate_action_pick_cover_from_disk) },
-	{ "SearchCoverFromWeb", NULL,
+	{ "SearchCoverFromWeb", GTK_STOCK_FIND,
 	  N_("_Search on Internet"), NULL,
 	  N_("Search for a disc cover on Internet"),
 	  G_CALLBACK (activate_action_search_cover_on_internet) },
